/*
 * HELP.C - Help for Msged/Q
 *
 * Msged/Q message editor for QuickBBS  Copyright 1990 by P.J. Muller
 *
 */

#include <string.h>
#include <stdlib.h>

#include "msged.h"
#include "screen.h"

#ifdef MINI		/* no help in mini version */

void help() {}
void helpedit() {}
void helparea() {}
void helplist() {}

#else	/* not MINI */

/*
 * Display a help page
 * Order of commands: -|^#
 */

static void helppage(char **msg)
{
  char *t;
  BYTE x=1, y=1, ytop=1;

  cls();
  while (*msg != NULL) {
    t = *msg++;

    if (*t == '-') {		/* store y position for '|' */
      ytop = y;  t++;
    } /* if */

    if (*t == '|') {		/* second column */
      x = maxx/2+1;  y = ytop;  t++;
    } /* if */

    if (*t == '^') {		/* highlight */
      set_color(co_hilite);
      t++;
    } else set_color(co_normal);

    if (*t == '#') {		/* center */
      t++;
      gotoxy((maxx-strlen(t))/2, y);
    } else gotoxy(x,y);

    bputs(t);
    y++;
  } /* while */

  gotoxy(1,25);
  bputs("Press any key to continue....");
  video_update();
  getkey();
} /* helppage */

/*
 * Message reading help
 */

void help()
{
  static char *help[]={

"^# Msged/Q Message Reading Help ",
"",
"-^Message Movement",
"",
"Right/Left  Next/Previous message",
"Enter       Move on in same direction",
"Ctrl-Left   Follow message thread down",
"Ctrl-Right  Follow message thread up",
"Home        First message in thread",
"End         Last message read",
"Ctrl-Home   First message in area",
"Ctrl-End    Last message in area",
"Ctrl-N/P    Next/Prev area",
"+/-         Next/Prev newmail area",
"Alt-F       Find string in text/headers",
"Alt-L       List headers and move",
"Alt-Z       Find string in headers",
"",
"^Message Scrolling",
"",
"Up/Down     Scroll message by lines",
"PgUp/PgDn   Scroll message by pages",
"Space       Page down",
"|^Message Operations",
"",
"Alt-C      Change message",
"Alt-D      Delete message",
"Alt-E      Enter message",
"Alt-M      Move, Copy and Forward",
"Alt-Q      Quote message in area",
"Alt-N      Quote and move",
"Alt-R      Reply to message",
"Alt-W      Write message to file",
"",
"^Miscellaneous",
"",
"Alt-A      Select area",
"Alt-G      Global area options",
#ifndef MINI
"Alt-O      Operating system shell",
#endif
#ifdef SETTINGS
"Alt-S      Settings screen",
#endif
"Alt-U      Update high mark",
"Alt-V      View hidden lines (toggle)",
"Alt-H      This screen",
"Alt-X/Esc  Exit Msged/Q",
NULL};

  helppage(help);
} /* help */

/*
 * Message editing help
 */

void helpedit()
{
  static char *help[]={

"^# Msged/Q Message Editing Help ",
"",
"-^Moving",
"",
"Right/Left  Next/Previous character",
"Ctrl-Right  Next word",
"Ctrl-Left   Previous word",
"Home        Start of line",
"End         End of line",
"PgDn        Next page",
"PgUp        Previous page",
"Ctrl-PgUp   Start of page",
"Ctrl-PgDn   End of page",
"Ctrl-Home   Start of message",
"Ctrl-End    End of message",
"",
"^Block commands",
"",
"Alt-A       Anchor a block",
"Alt-C       Cut a block",
"Alt-P       Paste a block",
"|^Insert and Delete",
"",
"Ins         Toggle insert mode",
"Del         Delete current character",
"BackSpace   Delete previous character",
"Alt-K       Delete to end of line",
"Alt-D       Delete line",
"Enter       Break a line",
"",
"^Miscellaneous",
"",
"Esc         Abort message",
"Alt-S       Save message",
#ifndef MINI
"Alt-O       Operating system shell",
#endif
"Alt-I       Import text file",
"Alt-W       Write message to text file",
NULL};

  helppage(help);
} /* helpedit */

/*
 * Message listing help
 */

void helplist()
{
  static char *help[]={

"^# Msged/Q Message List Help ",
"",
"^Moving",
"",
"Up          Move up",
"Down        Move down",
"PgDn        Next page",
"PgUp        Previous page",
"Ctrl-PgUp   Start of list",
"Ctrl-PgDn   End of list",
"",
"^Miscellaneous",
"",
"Alt-A       Toggle netmail addresses on/off",
NULL};

  helppage(help);
} /* helplist */

/*
 * Message areas help
 */

void helparea()
{
  static char *help[]={

"^# Msged/Q Area Screen Help ",
"",
"",
"",
"-^Moving around",
"",
"Right   Next column",
"Left    Previous column",
"Down    Next line",
"Up      Previous line",
"Home    First area",
"End     Last area",
"PgDn    Down column",
"PgUp    Up column",
"+/-     Next/Prev newmail area",
"",
"Any other character: Incremental Search",
"|^Miscellaneous",
"",
"Enter   Select new area",
"Esc     Select original area",
"Alt-T   Toggle descriptions/tags",
"Alt-X   Exit Msged/Q",
NULL};

  helppage(help);
} /* helparea */

#endif /* MINI */
