/*
 * QUICKBBS.H - QuickBBS structures
 *
 * Msged/Q message editor for QuickBBS  Copyright 1990 by P.J. Muller
 *
 */

typedef struct {
				/* msgattr byte */
   WORD  is_del:1;		/* Deleted */
   WORD  is_transit:1;		/* Unmoved outgoing net message */
   WORD  is_netm:1;		/* Net mail message */
   WORD  is_priv:1;		/* private */
   WORD  is_rcvd:1;		/* Received */
   WORD  is_echotr:1;		/* Unmoved outgoing echo message */
   WORD  is_local:1;		/* local bit */
   WORD  is_reserved1:1;	/* reserved */

				/* netattr byte */
   WORD  is_kill:1;		/* Kill after sent bit */
   WORD  is_sent:1;		/* sent ok */
   WORD  is_file:1;		/* file(s) attached */
   WORD  is_crash:1;		/* crash priority */
   WORD  is_reqrec:1;		/* Request receipt */
   WORD  is_audreq:1;		/* Audit request */
   WORD  is_retrec:1;		/* Is a return receipt */
   WORD  is_reserved2:1;	/* reserved */
} MSGBITS;

/* Note that string fields in *.BBS files must be converted to Pascal strings
   before being written out */

typedef struct {		/* MSGHDR.BBS record */
   int   msgnum,
	 reply,			/* thread to previous message */
	 up,			/* thread to next message */
	 times;			/* number of times read */
   WORD  startrec;		/* pointer into MSGTXT.BBS */
   int   numrecs,		/* number of 256-byte records in MSGTXT.BBS */
	 dest_net,		/* destination net */
	 dest,			/* destination node */
	 orig_net,		/* originating net */
	 orig;			/* originating node */
   BYTE  destzone,		/* destination zone (not in plain msged) */
	 origzone;		/* originating zone (not in plain msged) */
   int   cost;			/* actual cost of this msg */

   MSGBITS bits;		/* msg and net attributes */

   BYTE  board;			/* QuickBBS board number */
   char  posttime[6];		/* Post time */
   char  postdate[9];		/* Post date */
   char  to[36],		/* Who to */
	 from[36],		/* Who from */
	 subj[73];		/* message subject */
} MSGHEADER;

#define BLIM 200		/* QuickBBS message board limit */

/* -------------------------------------------------------------------- */

typedef struct {  /* Used in the MSGINFO.BBS file */

   int   lowmsg;              /* Lowest Message in File  */
   int   highmsg;             /* Highest Message in File */

   int   totalactive;         /* Total Active Messages   */

   int   activemsgs[BLIM];

} INFORECORD;

/* -------------------------------------------------------------------- */

typedef struct {  /* Used in the MSGIDX.BBS file */

   int   msgnum;
   BYTE  board;

} IDXRECORD;

/* -------------------------------------------------------------------- */

typedef struct {  /* Part of Configuration Record */

   BYTE        status;       /* 0=Deleted 1=Enabled 2=Disabled */
   char        runtime[6];
   BYTE        errorlevel;
   BYTE        days;
   BOOLEAN     forced;
   char        lasttimerun[9];

} EVENTRECORD;

/* -------------------------------------------------------------------- */
typedef BYTE flagtype[4];

typedef struct {  /* Part of Configuration Record */

   char     name[17];
   BYTE     typ;        /* 0=Standard 1=Net 2=EMail 3=Echo */
   BYTE     kinds;      /* 0=Both 1=Pvt 2=Pub 3=Read-Only */

   BOOLEAN  combined;
   BOOLEAN  aliases;

   int      readseclvl;
   flagtype readflags;

   int      writeseclvl;
   flagtype writeflags;

   int      sysopseclvl;
   flagtype sysopflags;

} BOARDRECORD;

/* -------------------------------------------------------------------- */

typedef struct {  /* Used for the CONFIG.BBS file */

   /*  Modem Parameters  */
   int      commport,
            initbaud,
            inittimes,
            answerwait;
   char     modeminitstr[71],
            modembusystr[71],
            modeminitresp[41],
            modembusyresp[41],
            resp300[41],
            resp1200[41],
            resp2400[41];

   /*  System Paths  */
   char     menupath[67],
            textpath[67],
            netpath[67];

   /*  Restriction Parameters  */
   int      minbaud,
            graphicsbaud,
            xferbaud;
   char     lowbaudstart[6],
            lowbaudend[6],
            downloadstart[6],
            downloadend[6],
            pagingstart[6],
            pagingend[6];

   /*  Matrix Information  */
   int      matrixzone,
            matrixnet,
            matrixnode,
            akanet[5],
            akanode[5],
            netmailboard;

   /*  Default Information for New Users  */
   int      defaultsec;
   int      defaultcredit;
   flagtype defaultflags;

   /*  Sysop Security Levels  */
   char     editorcmdstr[71];
   char     originline[61];
   char     sysopname[36];
   BOOLEAN  autologonchar,
            fastlogon,
            screenblanking,
            uselastread,
            monomode,
            directwrite,
            snowcheck,
            netechoexit,
            onewordnames,
            checkmail,
            askhomephone,
            askdataphone,
            graphicsavail;
   int      inactivetimeout,
            logontime,
            deffgcolor,
            defbgcolor,
            passwordtries,
            maxpagetimes,
            pagebelllen;

   BOOLEAN  use_xmodem,
	    use_xmodem1k,
	    use_ymodem,
	    use_ymodemg,
	    use_sealink,
	    use_zmodem,
	    inp_fields;
   char     quotestr[4];		/* QuickBBS quote string */
   int	    uploadcredit;
   char     loadingmessage[71];
   char     selectionprompt[71];
   WORD     versionid;
   char     resp4800[41];
   char     resp9600[41];
   int      akazone[5];
   int      matrixpoint;
   int      akapoint[5];
   BYTE     useaka[200];
   BOOLEAN  askage;
   char     systemname[41];
   DWORD    regkey;

   BYTE     extraspace[5];

   EVENTRECORD eventrec[30];
   BOARDRECORD boardrec[BLIM];

} CONFIGRECORD;

#define MAXAKA 5		/* from 0 to 5 */

/* -------------------------------------------------------------------- */

/* The 'attrib' word in the userrecord */
#define U_DELETED         0x01
#define U_CLS             0x02
#define U_MORE            0x04
#define U_ANSI            0x08
#define U_NOKILL          0x10
#define U_IGNOREDLHOURS   0x20
#define U_FSEDITOR        0x40
#define U_RESERVED        0x80

typedef struct {

   char  name[36],
         city[26],
         pwd[16],
         dataphone[13],
         homephone[13],
         lasttime[6],
         lastdate[9];
   BYTE attrib;
   flagtype flags;
   int   credit,
         pending,
         timesposted,
         highmsgread,
         seclvl,
         times,
         ups,
         downs,
         upk,
         downk,
         todayk,
         elapsed,
         len;
   BYTE extraspace[8];

} USERRECORD;
