/*
 * RA.H - RemoteAccess structures
 *
 * Msged/Q message editor for QuickBBS  Copyright 1990 by P.J. Muller
 *
 */

typedef BYTE RA_FLAG[4];

typedef struct {
  WORD zone;
  WORD net;
  WORD node;
  WORD point;
} RA_ADDRESS;

typedef char RA_TIME[6];
typedef BYTE RA_ASK;

/*
 * File: MESSAGES.RA
 *
 * MESSAGES contains all configuration data concerned with the
 * various message-areas in the RA message-base. One record is
 * provided for each message-area, and the file has a fixed-length
 * of 200 records. Contained in this file is information ranging
 * from the name of the area, to how the area can be accessed and
 * how the messages should be maintained (by RAMSG and the like).
 */

enum {RA_NumBoards=200};
enum RA_MsgTypes {RA_Local, RA_NetMail, RA_EchoMail};
enum RA_MsgKinds {RA_Both, RA_Private, RA_Public, RA_ROnly};

typedef struct {
  char name[41];
  BYTE type;		/* RA_MsgTypes */
  BYTE kinds;		/* RA_MsgKinds */
  BOOLEAN aliases;
  BYTE days_kill;
  BYTE recv_kill;
  WORD count_kill;

  WORD read_security;
  RA_FLAG read_flags;
  WORD write_security;
  RA_FLAG write_flags;
  WORD sysop_security;
  RA_FLAG sysop_flags;

  char origin[61];
  BYTE aka;
} RA_BOARDRECORD;

/*
Field descriptions:

     Name      :    Name of message-area, in readable form
     Typ       :    What type of messages should be kept in the
                    area. (Local, NetMail, EchoMail)
     MsgKinds  :    What kind of messages should be allowed in
                    the area. (Both, Private, Public, ROnly)
     Aliases   :    If used should be allowed to enter messages
                    under another name.
     DaysKill  :    How many days since posting messages should
                    be kept for
     RecvKill  :    How many days since posting messages
                    received should be kept for
     CountKill :    Maximum number of messages which should be
                    allowed in the message-area
     ReadSecurity:  Security value needed to read messages
     ReadFlags :    Flags needed to read messages
     WriteSecurity: Security value needed to post messages
     WriteFlags:    Flags needed to post messages
     SysopSecurity: Security value needed to perform 'Sysop'
                    functions on the message area. eg, "[D]elete
                    messages, [*] etc.."
     SysopFlags:    Flags needed to perform 'Sysop' functions on
                    the message area.
     OriginLine:    Text to place on " * Origin:" line, if empty
                    default one is used.
     AkaAddress:    Aka # address to use on " * Origin:" line,
                    MSGID's, EID's in EchoMail areas, and origin
                    addresses in the case of the message-area
                    being NetMail.
*/

/*
 * File: CONFIG.RA
 *
 * CONFIG contains most of the configuration information needed by
 * RA. It covers everything from the location of other data-files
 * to the restriction of various security levels from certain
 * functions to modem information.
 */

typedef struct {
  WORD version_id;
  BYTE comm_port;
  long baud;
  BYTE init_tries;
  char init_str[71];
  char busy_str[71];
  char init_resp[41];
  char busy_resp[41];
  char connect_300[41];
  char connect_1200[41];
  char connect_2400[41];
  char connect_4800[41];
  char connect_9600[41];
  char connect_19k[41];
  char connect_38k[41];
  BOOLEAN answer_phone;
  char ring[21];
  char answer_str[21];
  BOOLEAN flush_buffer;
  int modem_delay;
  int minimum_baud;
  int graphics_baud;
  int transfer_baud;
  RA_TIME slow_baud_time_start;
  RA_TIME slow_baud_time_end;
  RA_TIME download_time_start;
  RA_TIME download_time_end;
  RA_TIME paging_time_start;
  RA_TIME paging_time_end;
  char loading_msg[71];
  char list_prompt[71];
  int pwd_expiry;
  char menu_path[61];
  char text_path[61];
  char net_path[61];
  char nodelist_path[61];
  char msgbase_path[61];
  char sys_path[61];
  char external_ed_cmd[61];
  RA_ADDRESS address[10];	/* 0..9 */
  char system_name[31];
  int new_security;
  int new_credit;
  RA_FLAG new_flags;
  char origin_line[61];
  char quote_string[16];
  char sysop[36];
  char log_filename[61];
  BOOLEAN fast_logon;
  BOOLEAN allow_sys_rem;
  BOOLEAN mono_mode;
  BOOLEAN strict_pwd_checking;
  BOOLEAN direct_write;
  BOOLEAN snow_check;
  int credit_factor;
  int user_time_out;
  int logon_time;
  int password_tries;
  int max_page;
  int page_length;
  BOOLEAN check_for_multi_logon;
  BOOLEAN exclude_sysop_from_list;
  BOOLEAN one_word_names;
  RA_ASK check_mail;
  BOOLEAN ask_voice_phone;
  BOOLEAN ask_data_phone;
  BOOLEAN do_full_mail_check;
  BOOLEAN allow_file_shells;
  BOOLEAN fix_upload_dates;
  BOOLEAN show_file_dates;
  BOOLEAN ansi;
  BOOLEAN clear_screen;
  BOOLEAN more_prompt;
  BOOLEAN upload_msgs;
  BOOLEAN kill_sent;
  int crash_ask_sec;
  RA_FLAG crash_ask_flags;
  int crash_sec;
  RA_FLAG crash_flags;
  int fattach_sec;
  RA_FLAG fattach_flags;
  BYTE norm_fore;
  BYTE norm_back;
  BYTE stat_fore;
  BYTE stat_back;
  BYTE hi_back;
  BYTE hi_fore;
  BYTE wind_fore;
  BYTE wind_back;
  BYTE exit_local;
  BYTE exit_300;
  BYTE exit_1200;
  BYTE exit_2400;
  BYTE exit_4800;
  BYTE exit_9600;
  BYTE exit_19k;
  BYTE exit_38k;
  BOOLEAN multi_line;
  BYTE min_pwd_len;
  WORD min_up_space;
  RA_ASK hot_keys;
  BYTE border_fore;
  BYTE border_back;
  BYTE bar_fore;
  BYTE bar_back;
  BYTE log_style;
  BYTE multi_tasker;
  BYTE pwd_board;
  WORD buffer_size;
  char fkeys[10][61];	/* 1..10 actually */
  BYTE future_expansion[1024];
} RA_CONFIG;

/*
Field descriptions:

     VersionID :    May be used in future versions. Currently
                    set to $0000 (or 0x0000)
     CommPort  :    COMx: port number. 1 = COM1:
     Baud      :    bps rate to use with modem
     InitTries :    Number of times to try to initialise the
                    modem
     InitStr   :    Initialisation string
     BusyStr   :    Modem string to take modem off-hook
     InitResp  :    Initialisation response
     BusyResp  :    Response to BusyStr
     Connect300:    300bps connection response:
     Connect1200:   1200bps
     Connect2400:   2400bps
     Connect4800:   4800bps
     Connect9600:   9600bps
     Connect19k:    19.2kbps
     Connect38k:    38kbps
     AnswerPhone:   If RA should answer phone manually
     Ring      :    Modem "RING" code
     AnswerStr :    "ATA"-style modem command to answer phone
     FlushBuffer:   [not-used]
     ModemDelay:    Delay to pause between lines sent to modem
     MinimumBaud:   Minimum bps to allow a connection
     GraphicsBaud:  Minimum bps to allow ANSI graphics
                    capabilities
     TransferBaud:  Minimum bps to allow file-transfer
     SlowBaudTimeStart
               :    Time to start 300bps file-transfer window
     SlowBaudTimeEnd
               :    Time to stop 300bps file-transfer window
     DownloadTimeStart
               :    Time to start file-transfer window
     DownloadTimeEnd
               :    Time to stop file-transfer window
     PagingTimeStart
               :    Time to start Sysop-paging window
     PagingTimeEnd
               :    Time to stop Sysop-paging window
     LoadingMsg:    Message to display whilst loading external
                    program
     ListPrompt:    Prompt for 'list' menu-type
     PwdExpiry :    Number of calls to force password-change
     MenuPath  :    Path to *.MNU files
     TextPath  :    Path to *.ASC and *.ANS files
     NetPath   :    Path to netmail *.MSG files
     NodelistPath:  [not-used]
     MsgBasePath:   Path to MSG*.BBS files
     SysPath   :    Path to main RA 'root' system directory
     ExternalEdCmd: Command to execute external message-editor
     Address   :    Primary and AKA address info. Element '0' of
                    this array is the primary, elements 1-9 are
                    AKA's.
     SystemName:    Name of system
     NewSecurity:   New-user security
     NewCredit :    New-user netmail credit
     NewFlags  :    New-user flags
     OriginLine:    Default ' * Origin:'-line
     QuoteString:   String to proceed message quotes. '@' are
                    converted into initials of quotee
     Sysop     :    Name of Systems Operator
     LogFileName:   Filename of log-file
     FastLogon :    Should names be prompted in local mode? If
                    not, the Sysop's name is used.
     AllowSysRem:   Should the Sysop be allowed to log-on
                    remotely?
     MonoMode  :    Should RA use B&W colours?
     StrictPwdChecking
               :    Sets if RA should strictly compare new and
                    old passwords (using word-similarity
                    algorithm)
     DirectWrite:   Sets if RA should write directly to the
                    screen, otherwise BIOS calls are used
     SnowCheck :    Sets if RA should worry about possible snow
                    when writing to the screen
     CreditFactor:  Upload credit factor. A value of 'x' gives
                    a user 'x' more seconds online.
     UserTimeOut:   Number of seconds to decide a "User
                    inactivity"
     LogonTime :    Number of minutes allowed to get to the TOP
                    menu.
     PasswordTries: Number of tries a user can have at providing
                    a correct password
     MaxPage   :    Maximum number of Sysop-pages allowed per-
                    call
     PageLength:    Sysop-page length (in seconds)
     CheckForMultiLogon
               :    Specifies if RA should check to make sure a
                    user isn't online on more than one line at
                    once.
     ExcludeSysopFromList
               :    Tells RA if the Sysop should be placed on
                    user-lists etc..
     OneWordNames:  Specifies if users are allowed to log-on
                    with one-word names. eg "Phil"
     CheckMail :    Specifies if mail-checking should be done,
                    not done, or if the user should be prompted
                    before doing so.
     AskVoicePhone: Tells RA if new-users should be asked their
                    voice/home number
     AskDataPhone:  Tells RA if new-users should be asked their
                    data/business number
     DoFullMailCheck
               :    Specifies if RA should use 'short-cuts' on
                    mail-checking, by only checking onwards from
                    the highest message the user has read
                    previously.
     AllowFileShells:
               :    Controls the availability of 'shell'
                    control-codes in text files. This has been
                    a large security problem in the past, so is
                    optioned in the RA package.
     FixUploadDates
               :    Specifies if file-dates on uploads shouldbe
                    reset to the date/time they were sent
     ShowFileDates: Specifies if file-dates should be displayed
                    in files listings. More space is available
                    for descriptions without dates.
     ANSI      :    Controls new-user setting of ANSI graphics
     ClearScreen:   Controls new-user setting of screen-clearing
     MorePrompt:    Controls new-user setting of Continue[Y/n/=]
                    prompts
     UploadMsgs:    [not-used]
     KillSent  :    Controls use of 'Kill/Sent' on netmail
     CrashAskSec:   Minimum security to have option for 'crash'
                    netmail
     CrashAskFlags: Flags requried to have option for 'crash'
                    netmail
     CrashSec  :    Minimum security to force use of crash-mail
     CrashFlags:    Flags required to force use of crash-mail
     FAttachSec:    Security needed to have option for 'file-
                    attaches' on netmail
     FAttachFlags:  Flags needed to have option for 'file-
                    attaches' on netmail

               NOTE: The following attribute fields are for
               local control ONLY

     NormFore  :    Foreground for normal text
     NormBack  :    Background for normal text
     StatFore  :    Foreground for status-line
     StatBack  :    Background for status-line
     HiBack    :    Background for highlight text
     HiFore    :    Foreground for highlight text
     WindFore  :    Foreground for window text
     WindBack  :    Background for window text

          NOTE: The following values specify errorlevel numbers
          given-off the front-end-mailer for each bps rate.

     ExitLocal
     Exit300
     Exit1200
     Exit2400
     Exit4800
     Exit9600
     Exit19k
     Exit38k

     MultiLine :    Tells RA if it is running on a multi-line
                    system
     MinPwdLen :    Minimum password-length (characters)
     MinUpSpace:    Minimum disk space required for initiating
                    of upload
     HotKeys   :    Controls new-user setting of use of 'hot-
                    keys'
     BorderFore:    Foreground of window-border
     BorderBack:    Background of window-border
     BarFore   :    Foreground of menu-bar
     BarBack   :    Background of menu-bar
     LogStyle  :    Style of log RA should write
     MultiTasker:   Type of multi-tasker used (if any)
     PwdBoard  :    Msg area # to use for password failure
                    notification messages (WatchDog)
     BufferSize:    Internal buffer-size to use
     FKeys     :    Function-key command-lines
     FutureExpansion
               :    [not-used]
*/
